import route_vue_get from "../route/vue/get.js";

import {global} from "../global.js";

import storage from "../storage.js";

import alone from "../alone.js";

import safe from "../safe.js";

import http from "../http.js";

const pro = process || {}, env = pro.env || {};

export const app = {
    config: global(env.app_ver || "app" + "_config"),
    get_env(key = null) {
        return alone.get_arr(env, key, '')
    },
    get_conf(key = null, def = null) {
        return alone.get_arr(this.config.list().conf || {}, key, def);
    },
    get_origin() {
        return storage.get('originCache');
    },
    get_make() {
        return storage.get('makeCache');
    },
    http(path) {
        return http((import.meta.env.MODE === 'preview' ? '/api' : this.get_env('api'))).path(path).cross(true);
    },
    set_title(title = '') {
        return alone.set_title(title || this.config.get('title'));
    }
};

const self = {
    //本地设置的路由,和远程路由
    route: {local: [], web: []},
    //加载本地配置
    local_conf() {
        const local = pro.config || {data: null, random: null};
        const conf = ((local.data || '' && local.random || '') ? (safe.mov.de(local) || local) : local);
        if (alone.isset(conf)) {
            this.route.local = conf.route || [];
            app.config.object(alone.del_key(conf, 'route'));
        }
        return this;
    },
    //加载远程配置
    async web_conf() {
        const conf_file = app.get_env('conf');
        if (alone.isset(conf_file) && alone.is_string(conf_file)) {
            const res = await app.http(conf_file).query({
                data: safe.url.en({
                    make: app.get_make(),
                    origin: app.get_origin(),
                    domain: alone.domain.domain()
                })
            }).get(function (res) {
                if ((res.data || '') && (res.random || '')) {
                    return (safe.mov.de(res) || res);
                } else if (res.code == 200) {
                    return res.data;
                } else if (alone.is_object(res)) {
                    return res;
                }
                return false;
            }, function () {
                return false;
            });
            if (alone.isset(res)) {
                this.route.web = res.route || [];
                app.config.object(alone.del_key(res, 'route'));
            }
        }
        return this;
    },
    async load_file() {
        app.set_title();
        const ver = app.get_env('app_ver');
        //加载link css文件
        const load_link = function () {
            const link = app.config.get('file.link', []);
            if (alone.isset(link)) {
                const links = (alone.is_object(link) ? Object.values(link) : link);
                links.forEach((file) => (alone.set_link(file + '?v=' + ver)));
            }
        };
        //加载layui
        const load_lay = async function () {
            const lay = app.config.get('layui', {});
            if (lay.status || !lay.hasOwnProperty('status')) {
                alone.set_link((lay.link || './layui/css/layui.css') + '?v=' + ver);
                await alone.set_script((lay.script || './layui/layui.js') + '?v=' + ver);
                layui.config(Object.assign({
                    base: './model/',
                    version: ver
                }, lay.config || {})).extend(lay.extend || {});
                ((alone.isset(lay.use)) && await alone.lay_use(lay.use));
                window.$ = layui.$;
            }
        };
        //加载script js文件
        const load_script = async function () {
            const script = app.config.get('file.script', []);
            if (alone.isset(script)) {
                const scripts = (alone.is_object(script) ? Object.values(script) : script);
                for (const file of scripts) {
                    await alone.set_script(file + '?v=' + ver);
                }
            }
        };
        load_link();
        await load_lay();
        await load_script();
        return this;
    },
    //处理路由
    load_route() {
        const route_list = route_vue_get(app.config.get('routing', {}) || {}, this.route.local || [], this.route.web || []);
        app.config.list().route = route_list.route;
        this.route_config = route_list.routes || [];
        return this;
    },
    load_i18n() {
        this.i18n_config = {
            locale: app.config.get('locale'),
            fallback: app.config.get('fallback'),
            messages: app.config.get('lang'),
        };
        return this;
    },
    async process() {
        //有效时间
        const make_time = parseInt(app.config.get('make_time', 28800));
        //生成访问标识
        if (!alone.isset(app.get_make())) {
            storage.set('makeCache', alone.get_token(), make_time);
        }
        //保存第一次的来路
        if (!alone.isset(app.get_origin())) {
            const origin = alone.origin.get();
            if (alone.isset(origin)) {
                storage.set('originCache', origin, make_time);
            }
        }
        this.local_conf();
        await this.web_conf();
        await this.load_file();
        this.load_route().load_i18n();
    }
};

export const main = async function () {
    await self.process();
    return {route: self.route_config, i18n: self.i18n_config};
};